#include "doublespinbox.h"

#include <qspinbox.h>
#include <math.h>

/*!
  \class DoubleSpinBox doublespinbox.h
  \brief reimplements QSpinBox for double values
*/

DoubleSpinBox::DoubleSpinBox( QWidget* parent, const char* name ) : QSpinBox( int(-2e10), int(2e10), 10, parent, name )
{
	dVal = new QDoubleValidator( -2e8, 2e8, 2, this, "dVal" );
	setValidator( dVal );
	setValue( 0.0 );
	setFocusProxy(0);
}

DoubleSpinBox::DoubleSpinBox( double minValue, double maxValue, int decimals, double step, QWidget* parent, const char* name )
  : QSpinBox( int( minValue * pow( 10, decimals ) ), int( maxValue * pow( 10, decimals ) ), int( rint( step * pow( 10, decimals ) ) ), parent, name ), dec( decimals )
{
	dVal = new QDoubleValidator( minValue, maxValue, 2, this, "dVal" );
	setValidator( dVal );
	setValue( 0.0 );
}

DoubleSpinBox::~DoubleSpinBox()
{
}

QString DoubleSpinBox::mapValueToText ( int value )
{
	QString s;
	s.setNum( double( value )/ pow( 10, dec ), 'f', dec );
	return s;
}

int DoubleSpinBox::mapTextToValue ( bool * ok )
{
	return int( cleanText().toDouble( ok ) * pow( 10, dec ) );
}

double DoubleSpinBox::value() const
{
	return double( QRangeControl::value() ) / pow( 10, dec );
}

double DoubleSpinBox::minValue() const
{
	return double( QRangeControl::minValue() ) / pow( 10, dec );
}

double DoubleSpinBox::maxValue() const
{
	return double( QRangeControl::maxValue() ) / pow( 10, dec );
}

void DoubleSpinBox::setValue( double value )
{
	QRangeControl::setValue( int( value *  pow( 10, dec ) ) );
}

void DoubleSpinBox::setSteps( double value )
{
	QRangeControl::setSteps( int( value * pow( 10, dec ) ) , int( value * pow( 10, dec ) ) );
}

void DoubleSpinBox::updateValue( double value)
{
	QRangeControl::directSetValue( int( value * pow(10,dec) ) );
	updateDisplay();
}

void DoubleSpinBox::setRange( double minValue, double maxValue )
{
	QRangeControl::setRange( int( minValue *  pow( 10, dec ) ), int( maxValue *  pow( 10, dec ) ) );
	dVal->setRange( minValue, maxValue, 2 );
}

void DoubleSpinBox::valueChange()
{
	QSpinBox::valueChange();
	emit valueChanged( value() );
}

QSize DoubleSpinBox::sizeHint() const
{
	QFontMetrics fm = fontMetrics();
	int h = fm.height();
	if ( h < 12 )       // ensure enough space for the button pixmaps
	h = 12;
	int w = 35;         // minimum width for the value
	int wx = fm.width( "  " );
	QString s;
	s.setNum( minValue(), 'f', dec );
	s.prepend( prefix() );
	s.append( suffix() );
	w = QMAX( w, fm.width( s ) + wx );
	s.setNum( maxValue(), 'f', dec );
	s.prepend( prefix() );
	s.append( suffix() );
	w = QMAX( w, fm.width( s ) + wx );
	s = specialValueText();
	w = QMAX( w, fm.width( s ) + wx );

	QSize r( h // buttons AND frame both sides
		+ 6 // right/left margins
		+ w, // widest value
		frameWidth() * 2 // top/bottom frame
		+ 4 // top/bottom margins
		+ h // font height
		);
	return r;
}


bool DoubleSpinBox::eventFilter(QObject *object, QEvent *event) {
	if(event->type()==QEvent::MouseButtonPress) {
		if(((QMouseEvent*)event)->button()==LeftButton) {
			emit clicked();
		}
	}
	return QSpinBox::eventFilter(object,event);
}
